@@ -14,15 +14,17 @@
 # individuals. For the exact contribution history, see the revision
 # history and logs, available at http://trac.edgewall.org/log/.
 
-import re, sys, os, time
+import os, re, types
+
+from genshi.core import Markup
 
 from trac.core import *
 from trac.config import Option
 from trac.db.api import IDatabaseConnector, _parse_db_str
-from trac.db.util import ConnectionWrapper
+from trac.db.util import ConnectionWrapper, IterableCursor
 from trac.util import get_pkginfo
 from trac.util.compat import close_fds
-from trac.util.text import to_unicode
+from trac.util.text import exception_to_unicode, to_unicode
 from trac.util.translation import _
 
 _like_escape_re = re.compile(r'([/_%])')
@@ -50,14 +52,20 @@ try:
 except ImportError:
     has_mysqldb = False
 
+# Mapping from "abstract" SQL types to DB-specific types
+_type_map = {
+    'int64': 'bigint',
+}
+
 
 class MySQLConnector(Component):
-    """MySQL database support for version 4.1 and greater.
+    """Database connector for MySQL version 4.1 and greater.
     
-    Database urls should be of the form:
-        mysql://user[:password]@host[:port]/database
+    Database URLs should be of the form:
+    {{{
+    mysql://user[:password]@host[:port]/database
+    }}}
     """
-
     implements(IDatabaseConnector)
 
     mysqldump_path = Option('trac', 'mysqldump_path', 'mysqldump',
@@ -84,6 +92,7 @@ class MySQLConnector(Component):
                           MySQLdb.thread_safe())
             self.env.systeminfo.extend([('MySQL', mysql_info),
                                         ('MySQLdb', self._version)])
+            self.required = True
         return cnx
     
     def init_db(self, path, log=None, user=None, password=None, host=None,
@@ -113,7 +122,9 @@ class MySQLConnector(Component):
             name = '`%s`' % c
             table_col = filter((lambda x: x.name == c), table.columns)
             if len(table_col) == 1 and table_col[0].type.lower() == 'text':
-                if name == '`rev`':
+                if table_col[0].key_size is not None:
+                    name += '(%d)' % table_col[0].key_size
+                elif name == '`rev`':
                     name += '(20)'
                 elif name == '`path`':
                     name += '(255)'
@@ -131,6 +142,7 @@ class MySQLConnector(Component):
         coldefs = []
         for column in table.columns:
             ctype = column.type
+            ctype = _type_map.get(ctype, ctype)
             if column.auto_increment:
                 ctype = 'INT UNSIGNED NOT NULL AUTO_INCREMENT'
                 # Override the column type, as a text field cannot
@@ -149,12 +161,25 @@ class MySQLConnector(Component):
                   '_'.join(index.columns), table.name,
                   self._collist(table, index.columns))
 
+    def alter_column_types(self, table, columns):
+        """Yield SQL statements altering the type of one or more columns of
+        a table.
+        
+        Type changes are specified as a `columns` dict mapping column names
+        to `(from, to)` SQL type tuples.
+        """
+        alterations = []
+        for name, (from_, to) in sorted(columns.iteritems()):
+            to = _type_map.get(to, to)
+            if to != _type_map.get(from_, from_):
+                alterations.append((name, to))
+        if alterations:
+            yield "ALTER TABLE %s %s" % (table,
+                ', '.join("MODIFY %s %s" % each
+                          for each in alterations))
+
     def backup(self, dest_file):
-        try:
-            from subprocess import Popen, PIPE
-        except ImportError:
-            raise TracError('Python >= 2.4 or the subprocess module '
-                            'is required for pre-upgrade backup support')
+        from subprocess import Popen, PIPE
         db_url = self.env.config.get('trac', 'database')
         scheme, db_prop = _parse_db_str(db_url)
         db_name = os.path.basename(db_prop['path'])
@@ -169,13 +194,20 @@ class MySQLConnector(Component):
         args.extend(['-r', dest_file, db_name])
         
         environ = os.environ.copy()
-        environ['MYSQL_PWD'] = str(db_prop['password'])
-        p = Popen(args, env=environ, stderr=PIPE, close_fds=close_fds)
+        if 'password' in db_prop:
+            environ['MYSQL_PWD'] = str(db_prop['password'])
+        try:
+            p = Popen(args, env=environ, stderr=PIPE, close_fds=close_fds)
+        except OSError, e:
+            raise TracError(_("Unable to run %(path)s: %(msg)s",
+                              path=self.pg_dump_path,
+                              msg=exception_to_unicode(e)))
         errmsg = p.communicate()[1]
         if p.returncode != 0:
-            raise TracError("Backup attempt failed (%s)" % to_unicode(errmsg))
+            raise TracError(_("mysqldump failed: %(msg)s",
+                              msg=to_unicode(errmsg.strip())))
         if not os.path.exists(dest_file):
-            raise TracError("Backup attempt failed")
+            raise TracError(_("No destination file created"))
         return dest_file
 
 
@@ -194,11 +226,14 @@ class MySQLConnection(ConnectionWrapper):
             port = 3306
         cnx = MySQLdb.connect(db=path, user=user, passwd=password,
                               host=host, port=port, charset='utf8')
+        if hasattr(cnx, 'encoders'):
+            # 'encoders' undocumented but present since 1.2.1 (r422)
+            cnx.encoders[Markup] = cnx.encoders[types.UnicodeType]
         ConnectionWrapper.__init__(self, cnx, log)
         self._is_closed = False
 
     def cast(self, column, type):
-        if type == 'int':
+        if type == 'int'or type == 'int64':
             type = 'signed'
         elif type == 'text':
             type = 'char'
@@ -208,13 +243,22 @@ class MySQLConnection(ConnectionWrapper):
         return 'concat(%s)' % ', '.join(args)
 
     def like(self):
+        """Return a case-insensitive LIKE clause."""
         return "LIKE %s COLLATE utf8_general_ci ESCAPE '/'"
 
     def like_escape(self, text):
         return _like_escape_re.sub(r'/\1', text)
 
+    def quote(self, identifier):
+        """Return the quoted identifier."""
+        return "`%s`" % identifier
+
     def get_last_id(self, cursor, table, column='id'):
-        return self.cnx.insert_id()
+        return cursor.lastrowid
+
+    def update_sequence(self, cursor, table, column='id'):
+        # MySQL handles sequence updates automagically
+        pass
 
     def rollback(self):
         self.cnx.ping()
@@ -232,4 +276,4 @@ class MySQLConnection(ConnectionWrapper):
             self._is_closed = True
 
     def cursor(self):
-        return MySQLUnicodeCursor(self.cnx)
+        return IterableCursor(MySQLUnicodeCursor(self.cnx), self.log)
