@@ -10,8 +10,6 @@
 #
 # Author: Matthew Good <matt@matt-good.net>
 
-"""Syntax highlighting based on Pygments."""
-
 from datetime import datetime
 import os
 from pkg_resources import resource_filename
@@ -19,13 +17,14 @@ import re
 
 from trac.core import *
 from trac.config import ListOption, Option
+from trac.env import ISystemInfoProvider
 from trac.mimeview.api import IHTMLPreviewRenderer, Mimeview
 from trac.prefs import IPreferencePanelProvider
-from trac.util import get_module_path, get_pkginfo
+from trac.util import get_pkginfo
 from trac.util.datefmt import http_date, localtz
 from trac.util.translation import _
 from trac.web import IRequestHandler
-from trac.web.chrome import add_stylesheet
+from trac.web.chrome import add_notice, add_stylesheet
 
 from genshi import QName, Stream
 from genshi.core import Attrs, START, END, TEXT
@@ -43,9 +42,10 @@ __all__ = ['PygmentsRenderer']
 
 
 class PygmentsRenderer(Component):
-    """Syntax highlighting based on Pygments."""
+    """HTML renderer for syntax highlighting based on Pygments."""
 
-    implements(IHTMLPreviewRenderer, IPreferencePanelProvider, IRequestHandler)
+    implements(ISystemInfoProvider, IHTMLPreviewRenderer,
+               IPreferencePanelProvider, IRequestHandler)
 
     default_style = Option('mimeviewer', 'pygments_default_style', 'trac',
         """The default style to use for Pygments syntax highlighting.""")
@@ -53,7 +53,7 @@ class PygmentsRenderer(Component):
     pygments_modes = ListOption('mimeviewer', 'pygments_modes',
         '', doc=
         """List of additional MIME types known by Pygments.
-        
+
         For each, a tuple `mimetype:mode:quality` has to be
         specified, where `mimetype` is the MIME type,
         `mode` is the corresponding Pygments mode to be used
@@ -83,15 +83,18 @@ class PygmentsRenderer(Component):
 </html>"""
 
     def __init__(self):
+        self._types = None
+
+    # ISystemInfoProvider methods
+    
+    def get_system_info(self):
         version = get_pkginfo(pygments).get('version')
         # if installed from source, fallback to the hardcoded version info
         if not version and hasattr(pygments, '__version__'):
             version = pygments.__version__
-        self.env.systeminfo.append(('Pygments',version))
-                                        
-        self._types = None
-
-    # IHTMLPreviewRenderer implementation
+        yield 'Pygments', version
+    
+    # IHTMLPreviewRenderer methods
 
     def get_quality_ratio(self, mimetype):
         # Extend default MIME type to mode mappings with configured ones
@@ -117,10 +120,10 @@ class PygmentsRenderer(Component):
             raise Exception("No Pygments lexer found for mime-type '%s'."
                             % mimetype)
 
-    # IPreferencePanelProvider implementation
+    # IPreferencePanelProvider methods
 
     def get_preference_panels(self, req):
-        yield ('pygments', _('Pygments Theme'))
+        yield ('pygments', _('Syntax Highlighting'))
 
     def render_preference_panel(self, req, panel):
         styles = list(get_all_styles())
@@ -129,6 +132,7 @@ class PygmentsRenderer(Component):
             style = req.args.get('style')
             if style and style in styles:
                 req.session['pygments_style'] = style
+                add_notice(req, _('Your preferences have been saved.'))
             req.redirect(req.href.prefs(panel or None))
 
         output = self._generate('html', self.EXAMPLE)
@@ -138,7 +142,7 @@ class PygmentsRenderer(Component):
             'styles': styles
         }
 
-    # IRequestHandler implementation
+    # IRequestHandler methods
 
     def match_request(self, req):
         match = re.match(r'/pygments/(\w+)\.css', req.path_info)
@@ -182,6 +186,13 @@ class PygmentsRenderer(Component):
             name = aliases and aliases[0] or lexname
             for mimetype in mimetypes:
                 self._types[mimetype] = (name, self.QUALITY_RATIO)
+
+        # Pygments currently doesn't know application/javascript
+        if 'application/javascript' not in self._types:
+            js_entry = self._types.get('text/javascript')
+            if js_entry:
+                self._types['application/javascript'] = js_entry
+
         self._types.update(
             Mimeview(self.env).configured_modes_mapping('pygments')
         )
@@ -196,34 +207,42 @@ class GenshiHtmlFormatter(HtmlFormatter):
     of writing markup as strings to an output file.
     """
 
+    def _chunk(self, tokens):
+        """Groups tokens with the same CSS class in the token stream
+        and yields them one by one, along with the CSS class, with the
+        values chunked together."""
+
+        last_class = None
+        text = []
+        for ttype, value in tokens:
+            c = self._get_css_class(ttype)
+            if c == 'n':
+                c = ''
+            if c == last_class:
+                text.append(value)
+                continue
+
+            # If no value, leave the old <span> open.
+            if value:
+                yield last_class, u''.join(text)
+                text = [value]
+                last_class = c
+
+        if text:
+            yield last_class, u''.join(text)
+
     def generate(self, tokens):
         pos = (None, -1, -1)
         span = QName('span')
         class_ = QName('class')
 
         def _generate():
-            attrs = lc = None
-            text = []
-
-            for ttype, value in tokens:
-                c = self._get_css_class(ttype)
-                if c == 'n':
-                    c = ''
-                if c == lc:
-                    text.append(value)
-                elif value: # if no value, leave old span open
-                    if text:
-                        yield TEXT, u''.join(text), pos
-                    if attrs:
-                        yield END, span, pos
-                        attrs = None
-                    text = [value]
-                    lc = c
-                    if c:
-                        attrs = Attrs([(class_, c)])
-                        yield START, (span, attrs), pos
-            if text:
-                yield TEXT, u''.join(text), pos
-            if attrs:
-                yield END, span, pos
+            for c, text in self._chunk(tokens):
+                if c:
+                    attrs = Attrs([(class_, c)])
+                    yield START, (span, attrs), pos
+                    yield TEXT, text, pos
+                    yield END, span, pos
+                else:
+                    yield TEXT, text, pos
         return Stream(_generate())
