@@ -13,22 +13,20 @@
 # history and logs, available at http://trac.edgewall.org/log/.
 
 import os
-from StringIO import StringIO
 import tempfile
 import time
 import unittest
 
 from trac.config import *
 from trac.test import Configuration
+from trac.util import create_file
 
 
 class ConfigurationTestCase(unittest.TestCase):
 
-    if not hasattr(unittest.TestCase, "assertTrue"):
-        assertTrue = unittest.TestCase.failUnless   # Python 2.3 compatibility
-
     def setUp(self):
-        self.filename = os.path.join(tempfile.gettempdir(), 'trac-test.ini')
+        tmpdir = os.path.realpath(tempfile.gettempdir())
+        self.filename = os.path.join(tmpdir, 'trac-test.ini')
         self._write([])
         self._orig_registry = Option.registry
         Option.registry = {}
@@ -70,7 +68,8 @@ class ConfigurationTestCase(unittest.TestCase):
 
     def test_default_int(self):
         config = self._read()
-        self.assertRaises(ConfigurationError, config.getint, 'a', 'option', 'b')
+        self.assertRaises(ConfigurationError,
+                          config.getint, 'a', 'option', 'b')
         self.assertEquals(0, config.getint('a', 'option'))
         self.assertEquals(1, config.getint('a', 'option', '1'))
         self.assertEquals(1, config.getint('a', 'option', 1))
@@ -80,6 +79,20 @@ class ConfigurationTestCase(unittest.TestCase):
 
         self.assertEquals(2, config.getint('a', 'option'))
 
+    def test_default_float(self):
+        config = self._read()
+        self.assertRaises(ConfigurationError,
+                          config.getfloat, 'a', 'option', 'b')
+        self.assertEquals(0.0, config.getfloat('a', 'option'))
+        self.assertEquals(1.2, config.getfloat('a', 'option', '1.2'))
+        self.assertEquals(1.2, config.getfloat('a', 'option', 1.2))
+        self.assertEquals(1.0, config.getfloat('a', 'option', 1))
+
+        class Foo(object):
+            option_a = Option('a', 'option', '2.5')
+
+        self.assertEquals(2.5, config.getfloat('a', 'option'))
+
     def test_default_path(self):
         config = self._read()
         class Foo(object):
@@ -109,13 +122,25 @@ class ConfigurationTestCase(unittest.TestCase):
         self.assertEquals('y', config.get('b', u'öption2', 'y'))
 
     def test_read_and_getbool(self):
-        self._write(['[a]', 'option = yes'])
+        self._write(['[a]', 'option = yes', 'option2 = true',
+                     'option3 = eNaBlEd', 'option4 = on',
+                     'option5 = 1', 'option6 = 123', 'option7 = 123.456',
+                     'option8 = disabled', 'option9 = 0', 'option10 = 0.0'])
         config = self._read()
         self.assertEquals(True, config.getbool('a', 'option'))
         self.assertEquals(True, config.getbool('a', 'option', False))
-        self.assertEquals(False, config.getbool('b', 'option2'))
-        self.assertEquals(False, config.getbool('b', 'option2', False))
-        self.assertEquals(False, config.getbool('b', 'option2', 'disabled'))
+        self.assertEquals(True, config.getbool('a', 'option2'))
+        self.assertEquals(True, config.getbool('a', 'option3'))
+        self.assertEquals(True, config.getbool('a', 'option4'))
+        self.assertEquals(True, config.getbool('a', 'option5'))
+        self.assertEquals(True, config.getbool('a', 'option6'))
+        self.assertEquals(True, config.getbool('a', 'option7'))
+        self.assertEquals(False, config.getbool('a', 'option8'))
+        self.assertEquals(False, config.getbool('a', 'option9'))
+        self.assertEquals(False, config.getbool('a', 'option10'))
+        self.assertEquals(False, config.getbool('b', 'option_b'))
+        self.assertEquals(False, config.getbool('b', 'option_b', False))
+        self.assertEquals(False, config.getbool('b', 'option_b', 'disabled'))
 
     def test_read_and_getint(self):
         self._write(['[a]', 'option = 42'])
@@ -126,6 +151,16 @@ class ConfigurationTestCase(unittest.TestCase):
         self.assertEquals(25, config.getint('b', 'option2', 25))
         self.assertEquals(25, config.getint('b', 'option2', '25'))
 
+    def test_read_and_getfloat(self):
+        self._write(['[a]', 'option = 42.5'])
+        config = self._read()
+        self.assertEquals(42.5, config.getfloat('a', 'option'))
+        self.assertEquals(42.5, config.getfloat('a', 'option', 25.3))
+        self.assertEquals(0, config.getfloat('b', 'option2'))
+        self.assertEquals(25.3, config.getfloat('b', 'option2', 25.3))
+        self.assertEquals(25.0, config.getfloat('b', 'option2', 25))
+        self.assertEquals(25.3, config.getfloat('b', 'option2', '25.3'))
+
     def test_read_and_getlist(self):
         self._write(['[a]', 'option = foo, bar, baz'])
         config = self._read()
@@ -151,15 +186,34 @@ class ConfigurationTestCase(unittest.TestCase):
         self.assertEquals(['', 'bar', 'baz'],
                           config.getlist('a', 'option', keep_empty=True))
 
-    def test_getpath(self):
+    def test_read_and_choice(self):
+        self._write(['[a]', 'option = 2', 'invalid = d'])
         config = self._read()
-        config.set('a', 'path_a', '/somewhere/file.txt')
-        config.set('a', 'path_b', 'file.txt')
-        config.set('a', 'path_c', './file.txt')
-        self.assertEquals('/somewhere/file.txt', os.path.splitdrive(
-                config.getpath('a', 'path_a'))[1].replace('\\', '/'))
-        self.assertNotEquals('file.txt', config.getpath('a', 'path_b'))
-        self.assertEquals(config.getpath('a', 'path_b'),
+
+        class Foo(object):
+            option = ChoiceOption('a', 'option', ['Item1', 2, '3'])
+            other = ChoiceOption('a', 'other', [1, 2, 3])
+            invalid = ChoiceOption('a', 'invalid', ['a', 'b', 'c'])
+        
+            def __init__(self):
+                self.config = config
+        
+        foo = Foo()
+        self.assertEquals('2', foo.option)
+        self.assertEquals('1', foo.other)
+        self.assertRaises(ConfigurationError, getattr, foo, 'invalid')
+
+    def test_getpath(self):
+        base = os.path.dirname(self.filename)
+        config = self._read()
+        config.set('a', 'path_a', os.path.join(base, 'here', 'absolute.txt'))
+        config.set('a', 'path_b', 'thisdir.txt')
+        config.set('a', 'path_c', os.path.join(os.pardir, 'parentdir.txt'))
+        self.assertEquals(os.path.join(base, 'here', 'absolute.txt'),
+                          config.getpath('a', 'path_a'))
+        self.assertEquals(os.path.join(base, 'thisdir.txt'),
+                          config.getpath('a', 'path_b'))
+        self.assertEquals(os.path.join(os.path.dirname(base), 'parentdir.txt'),
                           config.getpath('a', 'path_c'))
 
     def test_set_and_save(self):
@@ -335,6 +389,42 @@ class ConfigurationTestCase(unittest.TestCase):
             self.assertEqual(True, 'a' in config)
         self._test_with_inherit(testcb)
 
+    def test_inherit_multiple(self):
+        class Foo(object):
+            option_b = Option('b', 'option2', 'default')
+        base = os.path.dirname(self.filename)
+        relsite1 = os.path.join('sub1', 'trac-site1.ini')
+        site1 = os.path.join(base, relsite1)
+        relsite2 = os.path.join('sub2', 'trac-site2.ini')
+        site2 = os.path.join(base, relsite2)
+        os.mkdir(os.path.dirname(site1))
+        create_file(site1, '[a]\noption1 = x\n'
+                           '[c]\noption = 1\npath1 = site1\n')
+        try:
+            os.mkdir(os.path.dirname(site2))
+            create_file(site2, '[b]\noption2 = y\n'
+                               '[c]\noption = 2\npath2 = site2\n')
+            try:
+                self._write(['[inherit]',
+                             'file = %s, %s' % (relsite1, relsite2)])
+                config = self._read()
+                self.assertEqual('x', config.get('a', 'option1'))
+                self.assertEqual('y', config.get('b', 'option2'))
+                self.assertEqual('1', config.get('c', 'option'))
+                self.assertEqual(os.path.join(base, 'site1'),
+                                 config.getpath('c', 'path1'))
+                self.assertEqual(os.path.join(base, 'site2'),
+                                 config.getpath('c', 'path2'))
+                self.assertEqual('',
+                                 config.getpath('c', 'path3'))
+                self.assertEqual(os.path.join(base, 'site4'),
+                                 config.getpath('c', 'path4', 'site4'))
+            finally:
+                os.remove(site2)
+                os.rmdir(os.path.dirname(site2))
+        finally:
+            os.remove(site1)
+            os.rmdir(os.path.dirname(site1))
 
     def _test_with_inherit(self, testcb):
         sitename = os.path.join(tempfile.gettempdir(), 'trac-site.ini')
