@@ -16,9 +16,8 @@
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
 from StringIO import StringIO
-from datetime import datetime
+from datetime import datetime, timedelta
 import re
-from time import localtime, strftime, time
 
 from genshi.builder import tag
 
@@ -30,18 +29,18 @@ from trac.mimeview import Context
 from trac.perm import IPermissionRequestor
 from trac.resource import *
 from trac.search import ISearchSource, search_to_sql, shorten_result
-from trac.util.compat import set, sorted
-from trac.util.datefmt import parse_date, utc, to_timestamp, to_datetime, \
-                              get_date_format_hint, get_datetime_format_hint, \
-                              format_date, format_datetime
-from trac.util.text import shorten_line, CRLF, to_unicode
-from trac.util.translation import _
+from trac.util import as_bool
+from trac.util.datefmt import parse_date, utc, to_utimestamp, \
+                              get_datetime_format_hint, format_date, \
+                              format_datetime, from_utimestamp
+from trac.util.text import CRLF
+from trac.util.translation import _, tag_
 from trac.ticket import Milestone, Ticket, TicketSystem, group_milestones
-from trac.ticket.query import Query, QueryModule
+from trac.ticket.query import QueryModule
 from trac.timeline.api import ITimelineEventProvider
 from trac.web import IRequestHandler, RequestDone
-from trac.web.chrome import add_link, add_notice, add_stylesheet, \
-                            add_warning, INavigationContributor
+from trac.web.chrome import add_link, add_notice, add_script, add_stylesheet, \
+                            add_warning, Chrome, INavigationContributor
 from trac.wiki.api import IWikiSyntaxProvider
 from trac.wiki.formatter import format_to
 
@@ -61,7 +60,7 @@ class TicketGroupStats(object):
         
         `title` is the display name of this group of stats (e.g.
           'ticket status').
-        `unit` is the display name of the units for these stats (e.g. 'hour').
+        `unit` is the units for these stats in plural form, e.g. _('hours')
         """
         self.title = title
         self.unit = unit
@@ -114,13 +113,16 @@ class TicketGroupStats(object):
                 self.done_percent += interval['percent']
                 self.done_count += interval['count']
 
-        # We want the percentages to add up to 100%.  To do that, we fudge the
-        # first interval that counts as "completed".  That interval is adjusted
-        # by enough to make the intervals sum to 100%.
+        # We want the percentages to add up to 100%. To do that, we fudge one
+        # of the intervals. If we're <100%, we add to the smallest non-zero
+        # interval. If we're >100%, we subtract from the largest interval.
+        # The interval is adjusted by enough to make the intervals sum to 100%.
         if self.done_count and total_percent != 100:
-            fudge_int = [i for i in self.intervals
-                         if i['overall_completion']][0]
             fudge_amt = 100 - total_percent
+            fudge_int = [i for i in sorted(self.intervals,
+                                           key=lambda k: k['percent'],
+                                           reverse=(fudge_amt < 0))
+                         if i['percent']][0]
             fudge_int['percent'] += fudge_amt
             self.done_percent += fudge_amt
 
@@ -204,7 +206,7 @@ class DefaultTicketGroupStatsProvider(Component):
             for s, cnt in cursor:
                 status_cnt[s] = cnt
 
-        stat = TicketGroupStats('ticket status', 'ticket')
+        stat = TicketGroupStats(_('ticket status'), _('tickets'))
         remaining_statuses = set(all_statuses)
         groups =  self._get_ticket_groups()
         catch_all_group = None
@@ -250,7 +252,7 @@ class DefaultTicketGroupStatsProvider(Component):
             stat.add_interval(group.get('label', group['name']), 
                               group_cnt, query_args,
                               group.get('css_class', group['name']),
-                              bool(group.get('overall_completion')))
+                              as_bool(group.get('overall_completion')))
         stat.refresh_calcs()
         return stat
 
@@ -328,14 +330,20 @@ class RoadmapModule(Component):
         return req.path_info == '/roadmap'
 
     def process_request(self, req):
-        milestone_realm = Resource('milestone')
         req.perm.require('MILESTONE_VIEW')
 
-        showall = req.args.get('show') == 'all'
+        show = req.args.getlist('show')
+        if 'all' in show:
+            show = ['completed']
 
         db = self.env.get_db_cnx()
-        milestones = [m for m in Milestone.select(self.env, showall, db)
+        milestones = Milestone.select(self.env, 'completed' in show, db)
+        if 'noduedate' in show:
+            milestones = [m for m in milestones
+                          if m.due is not None or m.completed]
+        milestones = [m for m in milestones
                       if 'MILESTONE_VIEW' in req.perm(m.resource)]
+
         stats = []
         queries = []
 
@@ -356,8 +364,7 @@ class RoadmapModule(Component):
         username = None
         if req.authname and req.authname != 'anonymous':
             username = req.authname
-        icshref = req.href.roadmap(show=req.args.get('show'), user=username,
-                                   format='ics')
+        icshref = req.href.roadmap(show=show, user=username, format='ics')
         add_link(req, 'alternate', icshref, _('iCalendar'), 'text/calendar',
                  'ics')
 
@@ -365,8 +372,9 @@ class RoadmapModule(Component):
             'milestones': milestones,
             'milestone_stats': stats,
             'queries': queries,
-            'showall': showall,
+            'show': show,
         }
+        add_stylesheet(req, 'common/css/roadmap.css')
         return 'roadmap.html', data, None
 
     # Internal methods
@@ -383,7 +391,8 @@ class RoadmapModule(Component):
         def get_priority(ticket):
             value = priorities.get(ticket['priority'])
             if value:
-                return int(value * 9 / len(priorities))
+                return int((len(priorities) + 8 * value - 9) /
+                       (len(priorities) - 1))
 
         def get_status(ticket):
             status = ticket['status']
@@ -392,7 +401,8 @@ class RoadmapModule(Component):
             elif status == 'assigned' or status == 'reopened':
                 return 'IN-PROCESS'
             elif status == 'closed':
-                if ticket['resolution'] == 'fixed': return 'COMPLETED'
+                if ticket['resolution'] == 'fixed':
+                    return 'COMPLETED'
                 else: return 'CANCELLED'
             else: return ''
 
@@ -405,7 +415,8 @@ class RoadmapModule(Component):
                  + ':' + escape_value(value)
             firstline = 1
             while text:
-                if not firstline: text = ' ' + text
+                if not firstline:
+                    text = ' ' + text
                 else: firstline = 0
                 buf.write(text[:75] + CRLF)
                 text = text[75:]
@@ -427,7 +438,7 @@ class RoadmapModule(Component):
                    % __version__)
         write_prop('METHOD', 'PUBLISH')
         write_prop('X-WR-CALNAME',
-                   self.config.get('project', 'name') + ' - ' + _('Roadmap'))
+                   self.env.project_name + ' - ' + _('Roadmap'))
         for milestone in milestones:
             uid = '<%s/milestone/%s@%s>' % (req.base_path, milestone.name,
                                             host)
@@ -436,11 +447,9 @@ class RoadmapModule(Component):
                 write_prop('UID', uid)
                 write_utctime('DTSTAMP', milestone.due)
                 write_date('DTSTART', milestone.due)
-                write_prop('SUMMARY', _('Milestone %(name)s') % {
-                    'name': milestone.name
-                })
-                write_prop('URL', req.base_url + '/milestone/' +
-                           milestone.name)
+                write_prop('SUMMARY', _('Milestone %(name)s',
+                                        name=milestone.name))
+                write_prop('URL', req.abs_href.milestone(milestone.name))
                 if milestone.description:
                     write_prop('DESCRIPTION', milestone.description)
                 write_prop('END', 'VEVENT')
@@ -456,9 +465,9 @@ class RoadmapModule(Component):
                 if milestone.due:
                     write_prop('RELATED-TO', uid)
                     write_date('DUE', milestone.due)
-                write_prop('SUMMARY', _('Ticket #%(num)s: %(summary)s') % {
-                    'num': ticket.id, 'summary': ticket['summary']
-                })
+                write_prop('SUMMARY', _('Ticket #%(num)s: %(summary)s',
+                                        num=ticket.id,
+                                        summary=ticket['summary']))
                 write_prop('URL', req.abs_href.ticket(ticket.id))
                 write_prop('DESCRIPTION', ticket['description'])
                 priority = get_priority(ticket)
@@ -473,7 +482,7 @@ class RoadmapModule(Component):
                                    (ticket.id,))
                     row = cursor.fetchone()
                     if row:
-                        write_utctime('COMPLETED', to_datetime(row[0], utc))
+                        write_utctime('COMPLETED', from_utimestamp(row[0]))
                 write_prop('END', 'VTODO')
         write_prop('END', 'VCALENDAR')
 
@@ -484,7 +493,6 @@ class RoadmapModule(Component):
         raise RequestDone
 
 
-
 class MilestoneModule(Component):
 
     implements(INavigationContributor, IPermissionRequestor, IRequestHandler,
@@ -518,7 +526,7 @@ class MilestoneModule(Component):
 
     def get_timeline_filters(self, req):
         if 'MILESTONE_VIEW' in req.perm:
-            yield ('milestone', _('Milestones'))
+            yield ('milestone', _('Milestones reached'))
 
     def get_timeline_events(self, req, start, stop, filters):
         if 'milestone' in filters:
@@ -528,11 +536,11 @@ class MilestoneModule(Component):
             # TODO: creation and (later) modifications should also be reported
             cursor.execute("SELECT completed,name,description FROM milestone "
                            "WHERE completed>=%s AND completed<=%s",
-                           (to_timestamp(start), to_timestamp(stop)))
+                           (to_utimestamp(start), to_utimestamp(stop)))
             for completed, name, description in cursor:
                 milestone = milestone_realm(id=name)
                 if 'MILESTONE_VIEW' in req.perm(milestone):
-                    yield('milestone', datetime.fromtimestamp(completed, utc),
+                    yield('milestone', from_utimestamp(completed),
                           '', (milestone, description)) # FIXME: author?
 
             # Attachments
@@ -545,7 +553,8 @@ class MilestoneModule(Component):
         if field == 'url':
             return context.href.milestone(milestone.id)
         elif field == 'title':
-            return tag('Milestone ', tag.em(milestone.id), ' completed')
+            return tag_('Milestone %(name)s completed',
+                        name=tag.em(milestone.id))
         elif field == 'description':
             return format_to(self.env, None, context(resource=milestone),
                              description)
@@ -566,8 +575,15 @@ class MilestoneModule(Component):
         add_link(req, 'up', req.href.roadmap(), _('Roadmap'))
 
         db = self.env.get_db_cnx() # TODO: db can be removed
-        milestone = Milestone(self.env, milestone_id, db)
         action = req.args.get('action', 'view')
+        try:
+            milestone = Milestone(self.env, milestone_id, db)
+        except ResourceNotFound:
+            if 'MILESTONE_CREATE' not in req.perm('milestone', milestone_id):
+                raise
+            milestone = Milestone(self.env, None, db)
+            milestone.name = milestone_id
+            action = 'edit' # rather than 'new' so that it works for POST/save
 
         if req.method == 'POST':
             if req.args.has_key('cancel'):
@@ -578,7 +594,7 @@ class MilestoneModule(Component):
             elif action == 'edit':
                 return self._do_save(req, db, milestone)
             elif action == 'delete':
-                self._do_delete(req, db, milestone)
+                self._do_delete(req, milestone)
         elif action in ('new', 'edit'):
             return self._render_editor(req, db, milestone)
         elif action == 'delete':
@@ -591,14 +607,13 @@ class MilestoneModule(Component):
 
     # Internal methods
 
-    def _do_delete(self, req, db, milestone):
+    def _do_delete(self, req, milestone):
         req.perm(milestone.resource).require('MILESTONE_DELETE')
 
         retarget_to = None
         if req.args.has_key('retarget'):
             retarget_to = req.args.get('target') or None
         milestone.delete(retarget_to, req.authname)
-        db.commit()
         add_notice(req, _('The milestone "%(name)s" has been deleted.',
                           name=milestone.name))
         req.redirect(req.href.roadmap())
@@ -614,8 +629,11 @@ class MilestoneModule(Component):
         
         milestone.description = req.args.get('description', '')
 
-        due = req.args.get('duedate', '')
-        milestone.due = due and parse_date(due, tzinfo=req.tz) or None
+        if 'due' in req.args:
+            due = req.args.get('duedate', '')
+            milestone.due = due and parse_date(due, req.tz, 'datetime') or None
+        else:
+            milestone.due = None
 
         completed = req.args.get('completeddate', '')
         retarget_to = req.args.get('target')
@@ -628,24 +646,27 @@ class MilestoneModule(Component):
             warnings.append(msg)
 
         # -- check the name
-        if new_name:
-            if new_name != old_name:
-                # check that the milestone doesn't already exists
-                # FIXME: the whole .exists business needs to be clarified
-                #        (#4130) and should behave like a WikiPage does in
-                #        this respect.
-                try:
-                    other_milestone = Milestone(self.env, new_name, db)
-                    warn(_('Milestone "%(name)s" already exists, please '
-                           'choose another name', name=new_name))
-                except ResourceNotFound:
-                    milestone.name = new_name
-        else:
-            warn(_('You must provide a name for the milestone.'))
+        # If the name has changed, check that the milestone doesn't already
+        # exist
+        # FIXME: the whole .exists business needs to be clarified
+        #        (#4130) and should behave like a WikiPage does in
+        #        this respect.
+        try:
+            new_milestone = Milestone(self.env, new_name, db)
+            if new_milestone.name == old_name:
+                pass        # Creation or no name change
+            elif new_milestone.name:
+                warn(_('Milestone "%(name)s" already exists, please '
+                       'choose another name.', name=new_milestone.name))
+            else:
+                warn(_('You must provide a name for the milestone.'))
+        except ResourceNotFound:
+            milestone.name = new_name
 
         # -- check completed date
         if 'completed' in req.args:
-            completed = completed and parse_date(completed, req.tz) or None
+            completed = completed and parse_date(completed, req.tz,
+                                                 'datetime') or None
             if completed and completed > datetime.now(utc):
                 warn(_('Completion date may not be in the future'))
         else:
@@ -660,15 +681,16 @@ class MilestoneModule(Component):
             milestone.update()
             # eventually retarget opened tickets associated with the milestone
             if 'retarget' in req.args and completed:
-                cursor = db.cursor()
-                cursor.execute("UPDATE ticket SET milestone=%s WHERE "
-                               "milestone=%s and status != 'closed'",
-                                (retarget_to, old_name))
+                @self.env.with_transaction()
+                def retarget(db):
+                    cursor = db.cursor()
+                    cursor.execute("UPDATE ticket SET milestone=%s WHERE "
+                                   "milestone=%s and status != 'closed'",
+                                   (retarget_to, old_name))
                 self.env.log.info('Tickets associated with milestone %s '
                                   'retargeted to %s' % (old_name, retarget_to))
         else:
             milestone.insert()
-        db.commit()
 
         add_notice(req, _('Your changes have been saved.'))
         req.redirect(req.href.milestone(milestone.name))
@@ -687,10 +709,16 @@ class MilestoneModule(Component):
         return 'milestone_delete.html', data, None
 
     def _render_editor(self, req, db, milestone):
+        # Suggest a default due time of 18:00 in the user's timezone
+        default_due = datetime.now(req.tz).replace(hour=18, minute=0, second=0,
+                                                   microsecond=0)
+        if default_due <= datetime.now(utc):
+            default_due += timedelta(days=1)
+        
         data = {
             'milestone': milestone,
-            'date_hint': get_date_format_hint(),
             'datetime_hint': get_datetime_format_hint(),
+            'default_due': default_due,
             'milestone_groups': [],
         }
 
@@ -704,6 +732,7 @@ class MilestoneModule(Component):
         else:
             req.perm(milestone.resource).require('MILESTONE_CREATE')
 
+        Chrome(self.env).add_wiki_toolbars(req)
         return 'milestone_edit.html', data, None
 
     def _render_view(self, req, db, milestone):
@@ -748,19 +777,24 @@ class MilestoneModule(Component):
             groups = []
             for field in ticket_fields:
                 if field['name'] == by:
-                    if field.has_key('options'):
+                    if 'options' in field:
                         groups = field['options']
+                        if field.get('optional'):
+                            groups.insert(0, '')
                     else:
                         cursor = db.cursor()
-                        cursor.execute("SELECT DISTINCT %s FROM ticket "
-                                       "ORDER BY %s" % (by, by))
+                        cursor.execute("""
+                            SELECT DISTINCT COALESCE(%s,'') FROM ticket
+                            ORDER BY COALESCE(%s,'')
+                            """ % (by, by))
                         groups = [row[0] for row in cursor]
 
             max_count = 0
             group_stats = []
 
             for group in groups:
-                group_tickets = [t for t in tickets if t[by] == group]
+                values = group and (group,) or (None, group)
+                group_tickets = [t for t in tickets if t[by] in values]
                 if not group_tickets:
                     continue
 
@@ -782,6 +816,8 @@ class MilestoneModule(Component):
                     percent = float(gstat.count) / float(max_count) * 100
                 gs_dict['percent_of_max_total'] = percent
 
+        add_stylesheet(req, 'common/css/roadmap.css')
+        add_script(req, 'common/js/folding.js')
         return 'milestone_view.html', data, None
 
     # IWikiSyntaxProvider methods
@@ -831,6 +867,26 @@ class MilestoneModule(Component):
         else:
             return desc
 
+    def resource_exists(self, resource):
+        """
+        >>> from trac.test import EnvironmentStub
+        >>> env = EnvironmentStub()
+        
+        >>> m1 = Milestone(env)
+        >>> m1.name = 'M1'
+        >>> m1.insert()
+        
+        >>> MilestoneModule(env).resource_exists(Resource('milestone', 'M1'))
+        True
+        >>> MilestoneModule(env).resource_exists(Resource('milestone', 'M2'))
+        False
+        """
+        db = self.env.get_read_db()
+        cursor = db.cursor()
+        cursor.execute("SELECT name FROM milestone WHERE name=%s",
+                       (resource.id,))
+        return bool(cursor.fetchall())
+
     # ISearchSource methods
 
     def get_search_filters(self, req):
@@ -851,10 +907,10 @@ class MilestoneModule(Component):
         for name, due, completed, description in cursor:
             milestone = milestone_realm(id=name)
             if 'MILESTONE_VIEW' in req.perm(milestone):
+                dt = (completed and from_utimestamp(completed) or
+                      due and from_utimestamp(due) or datetime.now(utc))
                 yield (get_resource_url(self.env, milestone, req.href),
-                       get_resource_name(self.env, milestone),
-                       datetime.fromtimestamp(
-                           completed or due or time(), utc),
+                       get_resource_name(self.env, milestone), dt,
                        '', shorten_result(description, terms))
         
         # Attachments
