@@ -1,10 +1,9 @@
-from trac.log import logger_factory
 from trac.mimeview import Context
 from trac.test import Mock, EnvironmentStub, MockPerm
-from trac.ticket.query import Query, QueryModule
+from trac.ticket.query import Query, QueryModule, TicketQueryMacro
+from trac.util.datefmt import utc
 from trac.web.href import Href
 from trac.wiki.formatter import LinkFormatter
-from trac.db.sqlite_backend import sqlite_version
 
 import unittest
 import difflib
@@ -33,7 +32,8 @@ class QueryTestCase(unittest.TestCase):
 
     def setUp(self):
         self.env = EnvironmentStub(default_data=True)
-        self.req = Mock(href=self.env.href, authname='anonymous')
+        self.db = self.env.get_db_cnx()
+        self.req = Mock(href=self.env.href, authname='anonymous', tz=utc)
         
     def tearDown(self):
         self.env.reset_db()
@@ -65,6 +65,17 @@ ORDER BY COALESCE(t.id,0)=0 DESC,t.id DESC""")
         sql, args = query.get_sql()
         self.assertEqualSQL(sql,
 """SELECT t.id AS id,t.summary AS summary,t.owner AS owner,t.type AS type,t.status AS status,t.priority AS priority,t.milestone AS milestone,t.reporter AS reporter,t.description AS description,t.time AS time,t.changetime AS changetime,priority.value AS priority_value
+FROM ticket AS t
+  LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
+ORDER BY COALESCE(t.id,0)=0,t.id""")
+        self.assertEqual([], args)
+        tickets = query.execute(self.req)
+
+    def test_all_ordered_by_id_from_unicode(self):
+        query = Query.from_string(self.env, u'order=id')
+        sql, args = query.get_sql()
+        self.assertEqualSQL(sql,
+"""SELECT t.id AS id,t.summary AS summary,t.owner AS owner,t.type AS type,t.status AS status,t.priority AS priority,t.milestone AS milestone,t.time AS time,t.changetime AS changetime,priority.value AS priority_value
 FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
 ORDER BY COALESCE(t.id,0)=0,t.id""")
@@ -126,7 +137,7 @@ ORDER BY COALESCE(t.version,'')='' DESC,COALESCE(version.time,0)=0 DESC,version.
 """SELECT t.id AS id,t.summary AS summary,t.owner AS owner,t.type AS type,t.status AS status,t.priority AS priority,t.component AS component,t.time AS time,t.changetime AS changetime,t.milestone AS milestone,priority.value AS priority_value
 FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
-WHERE COALESCE(t.milestone,'')=%s
+WHERE ((COALESCE(t.milestone,'')=%s))
 ORDER BY COALESCE(t.id,0)=0,t.id""")
         self.assertEqual(['milestone1'], args)
         tickets = query.execute(self.req)
@@ -174,7 +185,7 @@ ORDER BY COALESCE(priority.value,'')='',%(cast_priority)s,t.id""" % {
 """SELECT t.id AS id,t.summary AS summary,t.milestone AS milestone,t.owner AS owner,t.type AS type,t.status AS status,t.priority AS priority,t.time AS time,t.changetime AS changetime,priority.value AS priority_value
 FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
-WHERE COALESCE(t.milestone,'')!=%s
+WHERE ((COALESCE(t.milestone,'')!=%s))
 ORDER BY COALESCE(t.id,0)=0,t.id""")
         self.assertEqual(['milestone1'], args)
         tickets = query.execute(self.req)
@@ -187,7 +198,7 @@ ORDER BY COALESCE(t.id,0)=0,t.id""")
 """SELECT t.id AS id,t.summary AS summary,t.status AS status,t.owner AS owner,t.type AS type,t.priority AS priority,t.milestone AS milestone,t.time AS time,t.changetime AS changetime,priority.value AS priority_value
 FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
-WHERE COALESCE(t.status,'') IN (%s,%s,%s)
+WHERE (COALESCE(t.status,'') IN (%s,%s,%s))
 ORDER BY COALESCE(t.id,0)=0,t.id""")
         self.assertEqual(['new', 'assigned', 'reopened'], args)
         tickets = query.execute(self.req)
@@ -199,8 +210,8 @@ ORDER BY COALESCE(t.id,0)=0,t.id""")
 """SELECT t.id AS id,t.summary AS summary,t.owner AS owner,t.type AS type,t.status AS status,t.priority AS priority,t.milestone AS milestone,t.time AS time,t.changetime AS changetime,priority.value AS priority_value
 FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
-WHERE COALESCE(t.owner,'') %(like)s
-ORDER BY COALESCE(t.id,0)=0,t.id""" %  {'like': self.env.get_db_cnx().like()})
+WHERE ((COALESCE(t.owner,'') %(like)s))
+ORDER BY COALESCE(t.id,0)=0,t.id""" % {'like': self.env.get_db_cnx().like()})
         self.assertEqual(['%someone%'], args)
         tickets = query.execute(self.req)
 
@@ -211,8 +222,8 @@ ORDER BY COALESCE(t.id,0)=0,t.id""" %  {'like': self.env.get_db_cnx().like()})
 """SELECT t.id AS id,t.summary AS summary,t.owner AS owner,t.type AS type,t.status AS status,t.priority AS priority,t.milestone AS milestone,t.time AS time,t.changetime AS changetime,priority.value AS priority_value
 FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
-WHERE COALESCE(t.owner,'') NOT %(like)s
-ORDER BY COALESCE(t.id,0)=0,t.id""" %  {'like': self.env.get_db_cnx().like()})
+WHERE ((COALESCE(t.owner,'') NOT %(like)s))
+ORDER BY COALESCE(t.id,0)=0,t.id""" % {'like': self.env.get_db_cnx().like()})
         self.assertEqual(['%someone%'], args)
         tickets = query.execute(self.req)
 
@@ -223,8 +234,8 @@ ORDER BY COALESCE(t.id,0)=0,t.id""" %  {'like': self.env.get_db_cnx().like()})
 """SELECT t.id AS id,t.summary AS summary,t.owner AS owner,t.type AS type,t.status AS status,t.priority AS priority,t.milestone AS milestone,t.time AS time,t.changetime AS changetime,priority.value AS priority_value
 FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
-WHERE COALESCE(t.owner,'') %(like)s
-ORDER BY COALESCE(t.id,0)=0,t.id""" %  {'like': self.env.get_db_cnx().like()})
+WHERE ((COALESCE(t.owner,'') %(like)s))
+ORDER BY COALESCE(t.id,0)=0,t.id""" % {'like': self.env.get_db_cnx().like()})
         self.assertEqual(['someone%'], args)
         tickets = query.execute(self.req)
 
@@ -235,8 +246,8 @@ ORDER BY COALESCE(t.id,0)=0,t.id""" %  {'like': self.env.get_db_cnx().like()})
 """SELECT t.id AS id,t.summary AS summary,t.owner AS owner,t.type AS type,t.status AS status,t.priority AS priority,t.milestone AS milestone,t.time AS time,t.changetime AS changetime,priority.value AS priority_value
 FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
-WHERE COALESCE(t.owner,'') %(like)s
-ORDER BY COALESCE(t.id,0)=0,t.id""" %  {'like': self.env.get_db_cnx().like()})
+WHERE ((COALESCE(t.owner,'') %(like)s))
+ORDER BY COALESCE(t.id,0)=0,t.id""" % {'like': self.env.get_db_cnx().like()})
         self.assertEqual(['%someone'], args)
         tickets = query.execute(self.req)
 
@@ -244,13 +255,14 @@ ORDER BY COALESCE(t.id,0)=0,t.id""" %  {'like': self.env.get_db_cnx().like()})
         self.env.config.set('ticket-custom', 'foo', 'text')
         query = Query.from_string(self.env, 'foo=something', order='id')
         sql, args = query.get_sql()
+        foo = self.db.quote('foo')
         self.assertEqualSQL(sql,
-"""SELECT t.id AS id,t.summary AS summary,t.owner AS owner,t.type AS type,t.status AS status,t.priority AS priority,t.milestone AS milestone,t.time AS time,t.changetime AS changetime,priority.value AS priority_value,foo.value AS foo
+"""SELECT t.id AS id,t.summary AS summary,t.owner AS owner,t.type AS type,t.status AS status,t.priority AS priority,t.milestone AS milestone,t.time AS time,t.changetime AS changetime,priority.value AS priority_value,%s.value AS %s
 FROM ticket AS t
-  LEFT OUTER JOIN ticket_custom AS foo ON (id=foo.ticket AND foo.name='foo')
+  LEFT OUTER JOIN ticket_custom AS %s ON (id=%s.ticket AND %s.name='foo')
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
-WHERE COALESCE(foo.value,'')=%s
-ORDER BY COALESCE(t.id,0)=0,t.id""")
+WHERE ((COALESCE(%s.value,'')=%%s))
+ORDER BY COALESCE(t.id,0)=0,t.id""" % ((foo,) * 6))
         self.assertEqual(['something'], args)
         tickets = query.execute(self.req)
 
@@ -258,12 +270,14 @@ ORDER BY COALESCE(t.id,0)=0,t.id""")
         self.env.config.set('ticket-custom', 'foo', 'text')
         query = Query(self.env, group='foo', order='id')
         sql, args = query.get_sql()
+        foo = self.db.quote('foo')
         self.assertEqualSQL(sql,
-"""SELECT t.id AS id,t.summary AS summary,t.owner AS owner,t.type AS type,t.status AS status,t.priority AS priority,t.milestone AS milestone,t.time AS time,t.changetime AS changetime,priority.value AS priority_value,foo.value AS foo
+"""SELECT t.id AS id,t.summary AS summary,t.owner AS owner,t.type AS type,t.status AS status,t.priority AS priority,t.milestone AS milestone,t.time AS time,t.changetime AS changetime,priority.value AS priority_value,%s.value AS %s
 FROM ticket AS t
-  LEFT OUTER JOIN ticket_custom AS foo ON (id=foo.ticket AND foo.name='foo')
+  LEFT OUTER JOIN ticket_custom AS %s ON (id=%s.ticket AND %s.name='foo')
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
-ORDER BY COALESCE(foo.value,'')='',foo.value,COALESCE(t.id,0)=0,t.id""")
+ORDER BY COALESCE(%s.value,'')='',%s.value,COALESCE(t.id,0)=0,t.id""" %
+        ((foo,) * 7))
         self.assertEqual([], args)
         tickets = query.execute(self.req)
 
@@ -275,7 +289,7 @@ ORDER BY COALESCE(foo.value,'')='',foo.value,COALESCE(t.id,0)=0,t.id""")
 """SELECT t.id AS id,t.summary AS summary,t.owner AS owner,t.type AS type,t.status AS status,t.priority AS priority,t.milestone AS milestone,t.time AS time,t.changetime AS changetime,priority.value AS priority_value
 FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
-WHERE COALESCE(t.owner,'') IN (%s,%s)
+WHERE (COALESCE(t.owner,'') IN (%s,%s))
 ORDER BY COALESCE(t.id,0)=0,t.id""")
         self.assertEqual(['someone', 'someone_else'], args)
         tickets = query.execute(self.req)
@@ -288,7 +302,7 @@ ORDER BY COALESCE(t.id,0)=0,t.id""")
 """SELECT t.id AS id,t.summary AS summary,t.owner AS owner,t.type AS type,t.status AS status,t.priority AS priority,t.milestone AS milestone,t.time AS time,t.changetime AS changetime,priority.value AS priority_value
 FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
-WHERE COALESCE(t.owner,'') NOT IN (%s,%s)
+WHERE (COALESCE(t.owner,'') NOT IN (%s,%s))
 ORDER BY COALESCE(t.id,0)=0,t.id""")
         self.assertEqual(['someone', 'someone_else'], args)
         tickets = query.execute(self.req)
@@ -302,7 +316,7 @@ ORDER BY COALESCE(t.id,0)=0,t.id""")
 """SELECT t.id AS id,t.summary AS summary,t.owner AS owner,t.type AS type,t.status AS status,t.priority AS priority,t.milestone AS milestone,t.time AS time,t.changetime AS changetime,priority.value AS priority_value
 FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
-WHERE (COALESCE(t.owner,'') %(like)s OR COALESCE(t.owner,'') %(like)s)
+WHERE ((COALESCE(t.owner,'') %(like)s OR COALESCE(t.owner,'') %(like)s))
 ORDER BY COALESCE(t.id,0)=0,t.id""" % {'like': self.env.get_db_cnx().like()})
         tickets = query.execute(self.req)
 
@@ -339,6 +353,122 @@ ORDER BY COALESCE(t.id,0)=0,t.id""")
         self.assertEqual([], args)
         tickets = query.execute(self.req)
 
+    def test_constrained_by_time_range(self):
+        query = Query.from_string(self.env, 'created=2008-08-01..2008-09-01', order='id')
+        sql, args = query.get_sql(self.req)
+        self.assertEqualSQL(sql,
+"""SELECT t.id AS id,t.summary AS summary,t.time AS time,t.owner AS owner,t.type AS type,t.status AS status,t.priority AS priority,t.changetime AS changetime,priority.value AS priority_value
+FROM ticket AS t
+  LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
+WHERE (((%(cast_time)s>=%%s AND %(cast_time)s<%%s)))
+ORDER BY COALESCE(t.id,0)=0,t.id""" % {
+          'cast_time': self.env.get_db_cnx().cast('t.time', 'int64')})
+        self.assertEqual([1217548800000000L, 1220227200000000L], args)
+        tickets = query.execute(self.req)
+
+    def test_constrained_by_time_range_exclusion(self):
+        query = Query.from_string(self.env, 'created!=2008-08-01..2008-09-01', order='id')
+        sql, args = query.get_sql(self.req)
+        self.assertEqualSQL(sql,
+"""SELECT t.id AS id,t.summary AS summary,t.time AS time,t.owner AS owner,t.type AS type,t.status AS status,t.priority AS priority,t.changetime AS changetime,priority.value AS priority_value
+FROM ticket AS t
+  LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
+WHERE ((NOT (%(cast_time)s>=%%s AND %(cast_time)s<%%s)))
+ORDER BY COALESCE(t.id,0)=0,t.id""" % {
+          'cast_time': self.env.get_db_cnx().cast('t.time', 'int64')})
+        self.assertEqual([1217548800000000L, 1220227200000000L], args)
+        tickets = query.execute(self.req)
+
+    def test_constrained_by_time_range_open_right(self):
+        query = Query.from_string(self.env, 'created=2008-08-01..', order='id')
+        sql, args = query.get_sql(self.req)
+        self.assertEqualSQL(sql,
+"""SELECT t.id AS id,t.summary AS summary,t.time AS time,t.owner AS owner,t.type AS type,t.status AS status,t.priority AS priority,t.changetime AS changetime,priority.value AS priority_value
+FROM ticket AS t
+  LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
+WHERE ((%(cast_time)s>=%%s))
+ORDER BY COALESCE(t.id,0)=0,t.id""" % {
+          'cast_time': self.env.get_db_cnx().cast('t.time', 'int64')})
+        self.assertEqual([1217548800000000L], args)
+        tickets = query.execute(self.req)
+
+    def test_constrained_by_time_range_open_left(self):
+        query = Query.from_string(self.env, 'created=..2008-09-01', order='id')
+        sql, args = query.get_sql(self.req)
+        self.assertEqualSQL(sql,
+"""SELECT t.id AS id,t.summary AS summary,t.time AS time,t.owner AS owner,t.type AS type,t.status AS status,t.priority AS priority,t.changetime AS changetime,priority.value AS priority_value
+FROM ticket AS t
+  LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
+WHERE ((%(cast_time)s<%%s))
+ORDER BY COALESCE(t.id,0)=0,t.id""" % {
+          'cast_time': self.env.get_db_cnx().cast('t.time', 'int64')})
+        self.assertEqual([1220227200000000L], args)
+        tickets = query.execute(self.req)
+
+    def test_constrained_by_time_range_modified(self):
+        query = Query.from_string(self.env, 'modified=2008-08-01..2008-09-01', order='id')
+        sql, args = query.get_sql(self.req)
+        self.assertEqualSQL(sql,
+"""SELECT t.id AS id,t.summary AS summary,t.changetime AS changetime,t.owner AS owner,t.type AS type,t.status AS status,t.priority AS priority,t.time AS time,priority.value AS priority_value
+FROM ticket AS t
+  LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
+WHERE (((%(cast_changetime)s>=%%s AND %(cast_changetime)s<%%s)))
+ORDER BY COALESCE(t.id,0)=0,t.id""" % {
+          'cast_changetime': self.env.get_db_cnx().cast('t.changetime', 'int64')})
+        self.assertEqual([1217548800000000L, 1220227200000000L], args)
+        tickets = query.execute(self.req)
+
+    def test_constrained_by_keywords(self):
+        query = Query.from_string(self.env, 'keywords~=foo -bar baz',
+                                  order='id')
+        sql, args = query.get_sql()
+        self.assertEqualSQL(sql,
+"""SELECT t.id AS id,t.summary AS summary,t.keywords AS keywords,t.owner AS owner,t.type AS type,t.status AS status,t.priority AS priority,t.time AS time,t.changetime AS changetime,priority.value AS priority_value
+FROM ticket AS t
+  LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
+WHERE (((COALESCE(t.keywords,'') %(like)s AND COALESCE(t.keywords,'') NOT %(like)s AND COALESCE(t.keywords,'') %(like)s)))
+ORDER BY COALESCE(t.id,0)=0,t.id""" % {'like': self.env.get_db_cnx().like()})
+        self.assertEqual(['%foo%', '%bar%', '%baz%'], args)
+        tickets = query.execute(self.req)
+
+    def test_constrained_by_milestone_or_version(self):
+        query = Query.from_string(self.env, 'milestone=milestone1&or&version=version1', order='id')
+        sql, args = query.get_sql()
+        self.assertEqualSQL(sql,
+"""SELECT t.id AS id,t.summary AS summary,t.owner AS owner,t.type AS type,t.status AS status,t.priority AS priority,t.component AS component,t.time AS time,t.changetime AS changetime,t.version AS version,t.milestone AS milestone,priority.value AS priority_value
+FROM ticket AS t
+  LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
+WHERE ((COALESCE(t.milestone,'')=%s)) OR ((COALESCE(t.version,'')=%s))
+ORDER BY COALESCE(t.id,0)=0,t.id""")
+        self.assertEqual(['milestone1', 'version1'], args)
+        tickets = query.execute(self.req)
+
+    def test_equal_in_value(self):
+        query = Query.from_string(self.env, r'status=this=that&version=version1',
+                                  order='id')
+        sql, args = query.get_sql()
+        self.assertEqualSQL(sql,
+"""SELECT t.id AS id,t.summary AS summary,t.owner AS owner,t.type AS type,t.priority AS priority,t.milestone AS milestone,t.component AS component,t.status AS status,t.time AS time,t.changetime AS changetime,t.version AS version,priority.value AS priority_value
+FROM ticket AS t
+  LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
+WHERE ((COALESCE(t.status,'')=%s) AND (COALESCE(t.version,'')=%s))
+ORDER BY COALESCE(t.id,0)=0,t.id""")
+        self.assertEqual(['this=that', 'version1'], args)
+        tickets = query.execute(self.req)
+
+    def test_special_character_escape(self):
+        query = Query.from_string(self.env, r'status=here\&now|maybe\|later|back\slash',
+                                  order='id')
+        sql, args = query.get_sql()
+        self.assertEqualSQL(sql,
+"""SELECT t.id AS id,t.summary AS summary,t.status AS status,t.owner AS owner,t.type AS type,t.priority AS priority,t.milestone AS milestone,t.time AS time,t.changetime AS changetime,priority.value AS priority_value
+FROM ticket AS t
+  LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
+WHERE (COALESCE(t.status,'') IN (%s,%s,%s))
+ORDER BY COALESCE(t.id,0)=0,t.id""")
+        self.assertEqual(['here&now', 'maybe|later', 'back\\slash'], args)
+        tickets = query.execute(self.req)
+
     def test_repeated_constraint_field(self):
         like_query = Query.from_string(self.env, 'owner!=someone|someone_else',
                                        order='id')
@@ -353,7 +483,8 @@ ORDER BY COALESCE(t.id,0)=0,t.id""")
     def test_csv_escape(self):
         query = Mock(get_columns=lambda: ['col1'],
                      execute=lambda r,c: [{'id': 1,
-                                           'col1': 'value, needs escaped'}])
+                                           'col1': 'value, needs escaped'}],
+                     time_fields=['time', 'changetime'])
         content, mimetype = QueryModule(self.env).export_csv(
                                 Mock(href=self.env.href, perm=MockPerm()),
                                 query)
@@ -382,11 +513,49 @@ class QueryLinksTestCase(unittest.TestCase):
                          'field and constraints separated by a "="]</em>')
 
 
+class TicketQueryMacroTestCase(unittest.TestCase):
+
+    def assertQueryIs(self, content, query, kwargs, format):
+        qs, kw, f = TicketQueryMacro.parse_args(content)
+        self.assertEqual(query, qs)
+        self.assertEqual(kwargs, kw)
+        self.assertEqual(format, f)
+    
+    def test_owner_and_milestone(self):
+        self.assertQueryIs('owner=joe, milestone=milestone1',
+                           'owner=joe&milestone=milestone1',
+                           dict(col='status|summary', max='0', order='id'),
+                           'list')
+
+    def test_owner_or_milestone(self):
+        self.assertQueryIs('owner=joe, or, milestone=milestone1',
+                           'owner=joe&or&milestone=milestone1',
+                           dict(col='status|summary', max='0', order='id'),
+                           'list')
+    
+    def test_format_arguments(self):
+        self.assertQueryIs('owner=joe, milestone=milestone1, col=component|severity, max=15, order=component, format=compact',
+                           'owner=joe&milestone=milestone1',
+                           dict(col='status|summary|component|severity', max='15', order='component'),
+                           'compact')
+        self.assertQueryIs('owner=joe, milestone=milestone1, col=id|summary|component, max=30, order=component, format=table',
+                           'owner=joe&milestone=milestone1',
+                           dict(col='id|summary|component', max='30', order='component'),
+                           'table')
+
+    def test_special_char_escaping(self):
+        self.assertQueryIs(r'owner=joe|jack, milestone=this\&that\|here\,now',
+                           r'owner=joe|jack&milestone=this\&that\|here,now',
+                           dict(col='status|summary', max='0', order='id'),
+                           'list')
+        
+
 def suite():
     suite = unittest.TestSuite()
     suite.addTest(unittest.makeSuite(QueryTestCase, 'test'))
     suite.addTest(unittest.makeSuite(QueryLinksTestCase, 'test'))
+    suite.addTest(unittest.makeSuite(TicketQueryMacroTestCase, 'text'))
     return suite
 
 if __name__ == '__main__':
-    unittest.main()
+    unittest.main(defaultTest='suite')
