@@ -1,12 +1,62 @@
 from Cookie import SimpleCookie as Cookie
 import time
+from datetime import datetime
 import unittest
 
-from trac.core import TracError
-from trac.log import logger_factory
 from trac.test import EnvironmentStub, Mock
-from trac.web.href import Href
-from trac.web.session import DetachedSession, Session, PURGE_AGE, UPDATE_INTERVAL
+from trac.web.session import DetachedSession, Session, PURGE_AGE, \
+                             UPDATE_INTERVAL, SessionAdmin
+from trac.core import TracError
+
+
+def _prep_session_table(db, spread_visits=False):
+    """ Populate the session table with known values.
+
+    Return a tuple of lists: (auth_list, anon_list, all_list)
+    """
+    cursor = db.cursor()
+    cursor.execute("DELETE FROM session")
+    cursor.execute("DELETE FROM session_attribute")
+    last_visit_base = time.mktime(datetime(2010, 1, 1).timetuple())
+    visit_delta = spread_visits and 86400 or 0
+    auth_list, anon_list = [], []
+    for x in xrange(20):
+        sid = 'name%02d' % x
+        authenticated = int(x < 10)
+        last_visit = last_visit_base + (visit_delta * x)
+        val = 'val%02d' % x
+        data = (sid, authenticated, last_visit, val, val)
+        if authenticated:
+            auth_list.append(data)
+        else:
+            anon_list.append(data)
+        cursor.execute("INSERT INTO session VALUES (%s, %s, %s)",
+                       (sid, authenticated, last_visit))
+        cursor.execute("INSERT INTO session_attribute VALUES "
+                       "(%s, %s, 'name', %s)",
+                       (sid, authenticated, val))
+        cursor.execute("INSERT INTO session_attribute VALUES "
+                       "(%s, %s, 'email', %s)",
+                       (sid, authenticated, val))
+    all_list = auth_list + anon_list 
+    db.commit()
+    return (auth_list, anon_list, all_list)
+
+
+def get_session_info(db, sid):
+    cursor = db.cursor()
+    cursor.execute("SELECT DISTINCT s.sid, n.value, e.value "
+                   "  FROM session AS s "
+                   " LEFT JOIN session_attribute AS n ON (n.sid=s.sid "
+                   "  AND n.name = 'name') "
+                   " LEFT JOIN session_attribute AS e ON (e.sid=s.sid "
+                   "  AND e.name = 'email') "
+                   "WHERE s.sid = %s", (sid,))
+    rows = [r for r in cursor]
+    if rows:
+        return rows[0]
+    else:
+        return (None, None, None)
 
 
 class SessionTestCase(unittest.TestCase):
@@ -16,6 +66,9 @@ class SessionTestCase(unittest.TestCase):
         self.env = EnvironmentStub()
         self.db = self.env.get_db_cnx()
 
+    def tearDown(self):
+        self.env.reset_db()
+
     def test_new_session(self):
         """
         Verify that a session cookie gets sent back to the client for a new
@@ -161,9 +214,12 @@ class SessionTestCase(unittest.TestCase):
         Verify that old sessions get purged.
         """
         cursor = self.db.cursor()
+        cursor.execute("INSERT INTO session "
+                       "VALUES ('123456', 0, %s)",
+                       (0,))
         cursor.execute("INSERT INTO session "
                        "VALUES ('987654', 0, %s)",
-                       (time.time() - PURGE_AGE - 3600,))
+                       (int(time.time() - PURGE_AGE - 3600),))
         cursor.execute("INSERT INTO session_attribute VALUES "
                        "('987654', 0, 'foo', 'bar')")
         
@@ -207,6 +263,37 @@ class SessionTestCase(unittest.TestCase):
                        "authenticated=0")
         self.assertEqual(0, cursor.fetchone()[0])
 
+    def test_change_anonymous_session(self):
+        """
+        Verify that changing from one anonymous session to an inexisting
+        anonymous session creates the new session and doesn't carry over
+        variables from the previous session.
+        """
+
+        cursor = self.db.cursor()
+        cursor.execute("INSERT INTO session VALUES ('123456', 0, 0)")
+        cursor.execute("INSERT INTO session_attribute VALUES "
+                       "('123456', 0, 'foo', 'bar')")
+
+        incookie = Cookie()
+        incookie['trac_session'] = '123456'
+        req = Mock(authname='anonymous', base_path='/', incookie=incookie,
+                   outcookie=Cookie())
+        session = Session(self.env, req)
+        self.assertEqual({'foo': 'bar'}, session)
+        
+        session.get_session('7890')
+        session['baz'] = 'moo'
+        session.save()
+        self.assertEqual({'baz': 'moo'}, session)
+
+        cursor.execute("SELECT COUNT(*) FROM session WHERE sid='7890' AND "
+                       "authenticated=0")
+        self.assertEqual(1, cursor.fetchone()[0])
+        cursor.execute("SELECT name, value FROM session_attribute "
+                       "WHERE sid='7890' AND authenticated=0")
+        self.assertEqual([('baz', 'moo')], cursor.fetchall())
+
     def test_add_authenticated_session_var(self):
         """
         Verify that new variables are inserted into the 'session' table in the
@@ -240,9 +327,43 @@ class SessionTestCase(unittest.TestCase):
                        "WHERE sid='john' AND name='foo'") 
         self.assertEqual('baz', cursor.fetchone()[0])
 
+    def test_authenticated_session_independence_var(self):
+        """
+        Verify that an anonymous session with the same name as an authenticated
+        session doesn't interfere with the latter.
+        """
+        cursor = self.db.cursor()
+        cursor.execute("INSERT INTO session VALUES ('john', 1, 0)")
+        cursor.execute("INSERT INTO session_attribute VALUES "
+                       "('john', 1, 'foo', 'bar')")
+
+        cursor.execute("SELECT value FROM session_attribute "
+                       "WHERE sid='john' AND authenticated=1 AND name='foo'")
+        self.assertEqual('bar', cursor.fetchone()[0])
+
+        incookie = Cookie()
+        incookie['trac_session'] = 'john'
+        req = Mock(authname='anonymous', base_path='/', incookie=incookie,
+                   outcookie=Cookie())
+        session = Session(self.env, req)
+        self.assert_('foo' not in session)
+        session['foo'] = 'baz'
+        session.save()
+
+        cursor.execute("SELECT value FROM session_attribute "
+                       "WHERE sid='john' AND authenticated=1 AND name='foo'")
+        rows = cursor.fetchall()
+        self.assertEqual(1, len(rows))
+        self.assertEqual('bar', rows[0][0])
+        cursor.execute("SELECT value FROM session_attribute "
+                       "WHERE sid='john' AND authenticated=0 AND name='foo'")
+        rows = cursor.fetchall()
+        self.assertEqual(1, len(rows))
+        self.assertEqual('baz', rows[0][0])
+
     def test_delete_authenticated_session_var(self):
         """
-        Verify that modifying a variable updates the 'session' table accordingly
+        Verify that deleting a variable updates the 'session' table accordingly
         for an authenticated session.
         """
         cursor = self.db.cursor()
@@ -278,7 +399,8 @@ class SessionTestCase(unittest.TestCase):
         req = Mock(authname='anonymous', base_path='/', incookie=incookie,
                    outcookie=outcookie)
         session = Session(self.env, req)
-        session.save() # updating should not require modifications
+        session['modified'] = True
+        session.save() # updating does require modifications
 
         self.assertEqual(PURGE_AGE, outcookie['trac_session']['expires'])
 
@@ -288,7 +410,7 @@ class SessionTestCase(unittest.TestCase):
 
     def test_modify_detached_session(self):
         """
-        Verify that a modifying a variable in a session not associated with a
+        Verify that modifying a variable in a session not associated with a
         request updates the database accordingly.
         """
         cursor = self.db.cursor()
@@ -322,9 +444,85 @@ class SessionTestCase(unittest.TestCase):
                        "WHERE sid='john' AND name='foo'")
         self.assertEqual(0, cursor.fetchone()[0])
 
+    def test_session_admin_list(self):
+        auth_list, anon_list, all_list = _prep_session_table(self.db)
+        sess_admin = SessionAdmin(self.env)
+
+        # Verify the empty case
+        self.assertRaises(StopIteration, sess_admin._get_list([]).next)
+
+        self.assertEqual([i for i in sess_admin._get_list(['authenticated'])],
+                         auth_list)
+        self.assertEqual([i for i in sess_admin._get_list(['anonymous'])],
+                         anon_list)
+        self.assertEqual([i for i in sess_admin._get_list(['*'])], all_list)
+        self.assertEqual([i for i in sess_admin._get_list(['name00'])][0],
+                         auth_list[0])
+        self.assertEqual([i for i in sess_admin._get_list(['name10:0'])][0],
+                         anon_list[0])
+        self.assertEqual([i for i in sess_admin._get_list(['name00', 'name01',
+                                                           'name02'])],
+                         all_list[:3])
+            
+    def test_session_admin_add(self):
+        auth_list, anon_list, all_list = _prep_session_table(self.db)
+        sess_admin = SessionAdmin(self.env)
+        self.assertRaises(Exception, sess_admin._do_add, 'name00')
+        sess_admin._do_add('john')
+        result = get_session_info(self.db, 'john')
+        self.assertEqual(result, ('john', None, None))
+        sess_admin._do_add('john1', 'John1')
+        result = get_session_info(self.db, 'john1')
+        self.assertEqual(result, ('john1', 'John1', None))
+        sess_admin._do_add('john2', 'John2', 'john2@example.org')
+        result = get_session_info(self.db, 'john2')
+        self.assertEqual(result, ('john2', 'John2', 'john2@example.org'))
+
+    def test_session_admin_set(self):
+        auth_list, anon_list, all_list = _prep_session_table(self.db)
+        sess_admin = SessionAdmin(self.env)
+        self.assertRaises(TracError, sess_admin._do_set, 'name', 'nothere',
+                          'foo')
+        sess_admin._do_set('name', 'name00', 'john')
+        result = get_session_info(self.db, 'name00')
+        self.assertEqual(result, ('name00', 'john', 'val00'))
+        sess_admin._do_set('email', 'name00', 'john@example.org')
+        result = get_session_info(self.db, 'name00')
+        self.assertEqual(result, ('name00', 'john', 'john@example.org'))
+
+    def test_session_admin_delete(self):
+        auth_list, anon_list, all_list = _prep_session_table(self.db)
+        sess_admin = SessionAdmin(self.env)
+        sess_admin._do_delete('name00')
+        result = get_session_info(self.db, 'name00')
+        self.assertEqual(result, (None, None, None))
+        sess_admin._do_delete('nothere')
+        result = get_session_info(self.db, 'nothere')
+        self.assertEqual(result, (None, None, None))
+        auth_list, anon_list, all_list = _prep_session_table(self.db)
+        sess_admin._do_delete('anonymous')
+        result = [i for i in sess_admin._get_list(['*'])]
+        self.assertEqual(result, auth_list)
+
+    def test_session_admin_purge(self):
+        sess_admin = SessionAdmin(self.env)
+
+        auth_list, anon_list, all_list = \
+            _prep_session_table(self.db, spread_visits=True)
+        sess_admin._do_purge('2010-01-02')
+        result = [i for i in sess_admin._get_list(['*'])]
+        self.assertEqual(result, auth_list + anon_list)
+
+        auth_list, anon_list, all_list = \
+            _prep_session_table(self.db, spread_visits=True)
+        sess_admin._do_purge('2010-01-12')
+        result = [i for i in sess_admin._get_list(['*'])]
+        self.assertEqual(result, auth_list + anon_list[1:])
+
 
 def suite():
     return unittest.makeSuite(SessionTestCase, 'test')
 
+
 if __name__ == '__main__':
     unittest.main()
